import struct
from typing import Dict, Any, List
from ohre.misc import Log
import json
import os


class ResType:
    ELEMENT = 0
    RAW = 6
    INTEGER = 8
    STRING = 9
    STRARRAY = 10
    INTARRAY = 11
    BOOLEAN = 12
    COLOR = 14
    ID = 15
    THEME = 16
    PLURAL = 17
    FLOAT = 18
    MEDIA = 19
    PROF = 20
    PATTERN = 21
    SYMBOL = 23
    RES = 24

    def __init__(self, value):
        self.value = value

    def __str__(self):
        if self.value == ResType.ELEMENT:
            return "ELEMENT"
        elif self.value == ResType.RAW:
            return "RAW"
        elif self.value == ResType.INTEGER:
            return "INTEGER"
        elif self.value == ResType.STRING:
            return "String"
        elif self.value == ResType.STRARRAY:
            return "STRARRAY"
        elif self.value == ResType.INTARRAY:
            return "INTARRAY"
        elif self.value == ResType.BOOLEAN:
            return "BOOLEAN"
        elif self.value == ResType.COLOR:
            return "COLOR"
        elif self.value == ResType.ID:
            return "ID"
        elif self.value == ResType.THEME:
            return "THEME"
        elif self.value == ResType.PLURAL:
            return "PLURAL"
        elif self.value == ResType.FLOAT:
            return "FLOAT"
        elif self.value == ResType.MEDIA:
            return "MEDIA"
        elif self.value == ResType.PROF:
            return "PROF"
        elif self.value == ResType.PATTERN:
            return "PATTERN"
        elif self.value == ResType.SYMBOL:
            return "SYMBOL"
        elif self.value == ResType.RES:
            return "RES"
        else:
            return f"UNKNOWN{self.value}"


class KeyParam:
    TYPE_LANGUAGE = 0
    TYPE_REGION = 1
    TYPE_RESOLUTION = 2
    TYPE_ORIENTATION = 3
    TYPE_DEVICETYPE = 4
    TYPE_NIGHTMODE = 6
    TYPE_MCC = 7
    TYPE_MNC = 8

    def __init__(self, raw):
        self.raw = raw

    @property
    def key_type(self):
        return self.raw & 0xffffffff

    @property
    def value(self):
        return self.raw >> 32

    def __str__(self):
        key_type = self.key_type
        value = self.value
        if key_type == self.TYPE_ORIENTATION:
            return "vertical" if value == 0 else "horizontal"
        elif key_type == self.TYPE_NIGHTMODE:
            return "dark" if value == 0 else "light"
        elif key_type == self.TYPE_DEVICETYPE:
            return f"device{value}"
        elif key_type == self.TYPE_RESOLUTION:
            return f"dpi{value}"
        elif key_type == self.TYPE_LANGUAGE:
            return f"Language"
        elif key_type == self.TYPE_REGION:
            return f"Region"
        return f"keyType:{key_type} value:{value}"


class IdOffset:
    def __init__(self, raw):
        self.raw = raw

    @property
    def id(self):
        return self.raw & 0xFFFFFFFF

    @property
    def offset(self):
        return (self.raw >> 32) & 0xFFFFFFFF


class ResIndexBuf:
    def __init__(self, buf: bytes):
        self.pos: int = 0
        self.buf: bytes = buf
        self.header: Dict[str, Any] = self.read_header()
        self.limit_key_configs: Dict[int, List[str]] = self.get_limit_key_configs()
        self.id_set: Dict[str, int] = self.get_id_set()
        self.resource_items: Dict[str, Dict[str, Any]] = self.get_resources_items()

        self.resources_content: Dict[str, Any] = self._get_resources_content()

    def save_resource_to_json(self, path):
        with open(os.path.join(path, 'resource_index_content.json'), 'w', encoding='utf-8') as json_file:
            json.dump(self.resources_content, json_file, ensure_ascii=False, indent=4)

    def _get_resources_content(self) -> Dict[str, Any]:
        resource_content = {"header": self.header,
                            "limit_key_configs": self.limit_key_configs,
                            "id_set": self.id_set,
                            "resource_items": self.resource_items}
        Log.debug("Resources content constructed successfully.")
        return resource_content

    def read_header(self) -> Dict[str, Any]:
        version = self.buf[:128]
        try:
            file_size, limit_key_config_count = struct.unpack('<II', self.buf[128:136])
            version_str = version.decode('utf-8', 'ignore').rstrip('\x00')
            Log.info(f"{version_str}, {file_size}, and the limit_key_config_count is {limit_key_config_count}")
            return {"version": version_str,
                    "file_size": file_size,
                    "limit_key_config_count": limit_key_config_count
                    }
        except Exception as e:
            Log.error(f"Decoding header version failed: {e}")
            raise

    def get_limit_key_configs(self) -> Dict[int, List[str]]:
        limit_key_configs: Dict[int, List[str]] = {}
        offset = self.pos+136
        for i in range(self.header["limit_key_config_count"]):
            offset += 4
            k_offset = struct.unpack('<i', self.buf[offset:offset + 4])[0]
            offset += 4
            key_count = struct.unpack('<I', self.buf[offset:offset + 4])[0]
            offset += 4

            params = []
            if key_count == 0:
                params = ['base']
            else:
                for _ in range(key_count):

                    key_type = self.buf[offset:offset+4]
                    key_value = self.buf[offset+4:offset+8]

                    key_type = struct.unpack('I', key_type)[0]
                    key_value_ = struct.unpack(f'{len(key_value)}s', key_value)
                    try:
                        key_value = key_value_[0].decode('utf-8').rstrip('\x00')[::-1]
                    except:
                        Log.error(f"Key value decoding to UTF-8 failed in {self.__class__.__name__}")
                        key_value = key_value_[0].decode('utf-8', 'ignore').rstrip('\x00')[::-1]
                    offset += 8

                    params.append(f"{str(KeyParam(key_type))}: {str(key_value)}")
                    print(params)
            limit_key_configs[k_offset] = params
        Log.info(f"Parsed limit_key_configs with {len(limit_key_configs)} entries and final offset {offset}")
        self.pos = offset

        return limit_key_configs

    def get_id_set(self) -> Dict[str, int]:
        id_set_map = {}
        offset = self.pos
        try:
            for i in range(self.header["limit_key_config_count"]):
                offset += 4

                id_count = self.buf[offset:offset+4]
                id_count = struct.unpack('<i', id_count)[0]
                offset += 4
                for _ in range(id_count):
                    values = struct.unpack('<Q', self.buf[offset:offset + 8])[0]
                    values = IdOffset(values)
                    id_set_map[str(values.id)] = values.offset
                    offset += 8
            Log.info(f"The count of id_set_map is {len(id_set_map)} and the final offset is {offset}")
        except (KeyError, struct.error, AttributeError) as e:
            Log.error(f"Failed to parse id_set_map in {self.__class__.__name__}: {e}")
            raise

        self.pos = offset
        return id_set_map

    def get_resources_items(self) -> Dict[str, Dict[str, Any]]:
        id_set = self.id_set
        resource_item_dict: Dict[str, Dict[str, Any]] = {}
        offset = self.pos
        try:
            for rid, ridx in id_set.items():
                offset = ridx
                size = struct.unpack_from('<I', self.buf, offset)[0]
                offset += 4

                res_type = struct.unpack_from('<I', self.buf, offset)[0]
                offset += 4

                res_id = struct.unpack_from('<I', self.buf, offset)[0]
                offset += 4
                
                data_size = struct.unpack_from('<H', self.buf, offset)[0]
                offset += 2

                data_value = self.buf[offset:offset+data_size]
                offset += data_size

                name_size = struct.unpack_from('<H', self.buf, offset)[0]
                offset += 2

                name_value = self.buf[offset:offset + name_size]
                offset += name_size
                try:
                    data_value = data_value.decode('utf-8').strip('\x00')
                except:
                    Log.error(f"data_value decoding to UTF-8 failed in {self.__class__.__name__}")
                    data_value = data_value.decode('utf-8', 'ignore').strip('\x00')

                try:
                    name_value = name_value.decode('utf-8').strip('\x00')
                except:
                    Log.error(f"name_value decoding to UTF-8 failed in {self.__class__.__name__}")
                    name_value = name_value.decode('utf-8', 'ignore').strip('\x00')

                resource_item_dict[str(res_id)] = {
                    "file_size": size,
                    "file_type": str(ResType(res_type)),
                    "file_value": data_value,
                    "file_name": name_value
                }
            Log.info(f"The count of resource_item_dict is {len(resource_item_dict)} and the last offset is {offset}")

        except (struct.error, UnicodeDecodeError) as e:
            Log.error(f"Failed to parse resource items in {self.__class__.__name__}: {e}")
            raise

        return resource_item_dict



# The current version of ohre
__version__ = "0.0.1a"


import logging

from ohre.misc import Log
from ohre.core import oh_app
from ohre.core import oh_hap

Log.init_log("ohre_test", log_dir=".")
Log.debug(f"ohre __init__.py called")


def set_log_dir(log_dir: str):
    Log.init_log("ohre_test", log_dir)


def set_log_level(level: str):
    print(f"ohre setting log level {level}")
    g_log = Log.get_logger()
    if (level.lower() == "debug"):
        g_log.setLevel(logging.DEBUG)
    elif (level.lower() == "info"):
        g_log.setLevel(logging.INFO)
    elif (level.lower() == "warn"):
        g_log.setLevel(logging.WARNING)
    elif (level.lower() == "warning"):
        g_log.setLevel(logging.WARNING)
    elif (level.lower() == "error"):
        g_log.setLevel(logging.ERROR)
    else:
        print(f"ohre set log level ERROR, level not valid")


def set_log_print(print_flag: bool):
    print(f"ohre setting log console print flag {print_flag}")
    Log.set_debug_print_flag(print_flag)

set_log_print(False)


OHRE_HAP_ROOT_WHITE = {
    ".": ["pack.info", "module.json", "resources.index"]
}

OHRE_HAP_WHITE = {
    "resources/base/media/": ["*.png", "*.json", "*.jpg", "*.jpeg", "*.gif", "*.svg"]
}

OHRE_HAP_BLACK = {
    "*": ["*.proto",
          "*.o", "*.h", "*.cpp", "*.c", "*.cc",
          "*.java", "*.go", "*.cs"],
    ".git": ["*"],
    ".svn": ["*"]
}

OHRE_APP_WHITE = {
    ".": ["pack.info", "*.hap"]
}



import fnmatch
import hashlib
import io
import json
import os
import zipfile
from typing import Any, Dict, List

import yara

from ohre.misc import Log

HAP_EXTRACT_PREFIX = "hap_extract_"


class FileNotPresent(Exception):
    pass


class ParaNotValid(Exception):
    pass


def fname_in_pattern_list(fname, pattern_list: List) -> bool:
    for patt in pattern_list:
        if (fnmatch.fnmatch(fname, patt)):
            return True
    return False


def cal_md5(file_path) -> str:
    hash_md5 = hashlib.md5()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def cal_sha1(file_path) -> str:
    hash_sha1 = hashlib.sha1()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_sha1.update(chunk)
    return hash_sha1.hexdigest()


def extract_local_zip_to(zip_path: str, unzip_folder: str):
    try:
        with zipfile.ZipFile(zip_path, "r") as zip_ref:
            zip_ref.extractall(unzip_folder)
    except zipfile.BadZipFile:
        Log.warn(f"unzip {zip_path} to {unzip_folder}. Bad ZIP file")
        return False


class oh_package(object):
    def __init__(self, value):
        Log.debug(f"oh_package init {type(value)} {value}")
        self.file_path = ""
        self._md5 = None
        self._sha1 = None
        if (isinstance(value, str)):
            self.file_path = value
            self.package = zipfile.ZipFile(value, "r")
            self._md5 = cal_md5(value)
            self._sha1 = cal_sha1(value)
        elif (isinstance(value, zipfile.ZipFile)):
            self.package = value
        elif (isinstance(value, io.BytesIO)):
            self.package = zipfile.ZipFile(value, "r")
        else:
            Log.error(f"{self._sha1} ERROR! init oh_package failed, value type {type(value)} NOT supported")

        self.files = self.package.namelist()
        self.pack_info = None
        self.get_pack_info()

    @property
    def sha1(self):
        return self._sha1

    @property
    def md5(self):
        return self._md5

    def extract_all_to(self, unzip_folder: str):
        try:
            self.package.extractall(unzip_folder)
        except zipfile.BadZipFile:
            Log.warn(f"{self._sha1} Bad ZIP file, {self.file_path}")
            return False

    def get_files(self) -> List[str]:
        # return files's name
        return self.files

    def get_file(self, filename: str) -> bytes:
        try:
            return self.package.read(filename)
        except KeyError:
            raise FileNotPresent(filename)

    def get_md5(self, filename: str = "") -> str:
        if (filename == ""):
            return self._md5
        else:
            raise Exception("Not implemented")

    def get_sha1(self, filename: str = "") -> str:
        if (filename == ""):
            return self._sha1
        else:
            raise Exception("Not implemented")

    def apply_yara_rule(self, rule_str: str = "", rule_path: str = "", fname_pattern_list: List = [],
                        file_list: List = []) -> Dict[str, List]:
        # rule_str rule_path: yara rule str ot yara rule file path, specify one of them
        # file_list: if len==0, use all files to match file name pattern in fname_pattern_list
        # return : a dict representing files that match input yara rules
        # e.g. dict{file_0: [match_rule0, match_rule1], file_1: [match_rule1]}
        all_files = file_list if (len(file_list)) else self.get_files()
        Log.info(f"{self._sha1} apply_yara_rule: all files {len(all_files)} patt list {len(fname_pattern_list)}")
        # === yara rule
        Log.info(f"{self._sha1} apply_yara_rule: rule_str/rule_path len {len(rule_str)}/{len(rule_path)}", False)
        if (len(rule_str)):
            rules = yara.compile(source=rule_str)
        elif (len(rule_path)):
            rules = yara.compile(filepath=rule_path)
        else:
            raise ParaNotValid(f"{self._sha1} both rule_str and rule_path are empty")
        # === filter
        files_need = []
        if (len(fname_pattern_list)):
            for fname in all_files:
                if (fname_in_pattern_list(fname, fname_pattern_list)):
                    files_need.append(fname)
        else:
            files_need = all_files
        # === apply rule, scan start
        match_d = dict()
        for fname in files_need:
            matches = rules.match(data=self.get_file(fname))
            if (len(matches)):
                Log.debug(f"{self._sha1} matches: {type(matches)} {matches} fname {fname}")
                if(fname in match_d):
                    match_d[fname].extend(matches)
                else:
                    match_d[fname] = matches
        return match_d

    # === pack.info analysis START ===
    def get_bundle_name(self) -> str:
        if (self.pack_info is not None and
            "summary" in self.pack_info.keys() and
                "app" in self.pack_info["summary"].keys() and
                "bundleName" in self.pack_info["summary"]["app"].keys()):
            return self.pack_info["summary"]["app"]["bundleName"]
        else:
            Log.warn(f"{self._sha1} get bundle name failed")
            return ""

    def get_version(self) -> Dict:
        return self.pack_info["summary"]["app"]["version"]

    def get_version_name(self) -> str:
        return str(self.pack_info["summary"]["app"]["version"]["name"])

    def get_version_code(self) -> str:
        return str(self.pack_info["summary"]["app"]["version"]["code"])

    def get_pack_info(self) -> Dict:
        if (self.pack_info is not None):
            return self.pack_info
        ret = None
        for fname in self.files:
            Log.debug(f"{self.sha1} get_pack_info fname {fname}")
            if (fname == "pack.info"):
                json_string = self.get_file(fname).decode("utf-8", errors="ignore")
                Log.info(f"pack.info: {fname} {json_string}", False)
                ret = json.loads(json_string)
                self.pack_info = ret
                return ret
        Log.error(f"{self._sha1} pack.info not found")
        return None

    def get_taget_api_version(self) -> int:
        api_lvs = set()
        if ("summary" in self.pack_info and "modules" in self.pack_info["summary"]):
            for d_ in self.pack_info["summary"]["modules"]:
                if ("apiVersion" in d_ and "target" in d_["apiVersion"]):
                    target_api_version = d_["apiVersion"]["target"]
                    if (isinstance(target_api_version, int) and target_api_version > 0):
                        api_lvs.add(target_api_version)
        return min(api_lvs)

    def get_compatible_api_version(self) -> int:
        api_lvs = set()
        if ("summary" in self.pack_info and "modules" in self.pack_info["summary"]):
            for d_ in self.pack_info["summary"]["modules"]:
                if ("apiVersion" in d_ and "compatible" in d_["apiVersion"]):
                    target_api_version = d_["apiVersion"]["compatible"]
                    if (isinstance(target_api_version, int) and target_api_version > 0):
                        api_lvs.add(target_api_version)
        return max(api_lvs)

    def is_api_version_release(self) -> bool:
        not_release_hit = False
        release_hit = False
        if ("summary" in self.pack_info and "modules" in self.pack_info["summary"]):
            for d_ in self.pack_info["summary"]["modules"]:
                if ("apiVersion" in d_ and "releaseType" in d_["apiVersion"]):
                    if (str(d_["apiVersion"]["releaseType"]).lower() == "release"):
                        release_hit = True
                        Log.debug(f"api version release_hit {d_}")
                    else:
                        Log.debug(f"api version not_release_hit {d_}")
                        not_release_hit = True
        if (release_hit and (not not_release_hit)):
            return True
        else:
            return False

    def get_packages_device_type(self) -> List:
        # pack.info packages deviceType
        device_type_l = list()
        if ("packages" in self.pack_info):
            for d_ in self.pack_info["packages"]:
                if ("deviceType" in d_):
                    device_type_l.extend(d_["deviceType"])
        return sorted(list(set(device_type_l)))
    # === pack.info analysis END ===



import struct


class Version:
    def __init__(self, version):
        self.version = version

    @property
    def main_ver(self):
        return self.version & 0x0000ff

    @property
    def sub_ver(self):
        return (self.version >> 8) & 0x0000ff

    @property
    def feat_ver(self):
        return (self.version >> 16) & 0x0000ff

    @property
    def build_ver(self):
        return (self.version >> 24) & 0x0000ff

    def __str__(self):
        return f"{self.main_ver}.{self.sub_ver}.{self.feat_ver}.{self.build_ver}"



class AbcHeader:
    def __init__(self,buf):
        self.pos = 0
        self.buf = buf
        self.magic, self.check_sum, self.version, self.file_size, self.foreign_off, self.foreign_size,\
            self.num_classes, self.class_idx_off, self.num_lnps, self.lnp_idx_off, self.num_literal_arrays,\
         self.literal_array_idx_off, self.num_index_regions, self.index_section_off = self._parse_header()

    def _parse_header(self):
        offset = 0
        magic = self._read_string(offset,8)# Magic string. Must be 'P' 'A' 'N' 'D' 'A' '\0' '\0' '\0'
        print(magic, offset)

        offset+=8
        # Adler32
        check_sum = self._read_uint32(offset)
        print(check_sum, offset)

        offset+=4
        # Version of the format. Current version is 0002.
        version_value = self._read_uint32(offset)
        version = Version(version_value)
        print("Version: ",version)
        if not (version.main_ver >= 9 and version.sub_ver >= 0 and version.feat_ver >= 0 and version.build_ver >= 0):
            raise NotImplementedError(
                f"Unsupported ABC Version {version.main_ver}.{version.sub_ver}.{version.feat_ver}.{version.build_ver}")

        offset += 4
        # Size of the file in bytes.
        file_size = self._read_uint32(offset)
        print(file_size)

        # uint32_t
        foreign_off = self._read_uint32(offset + 4)
        print('foreign_off', foreign_off,offset + 4)

        # Size of the foreign region in bytes.
        # uint32_t
        foreign_size = self._read_uint32(offset + 8)
        print('foreign_size', foreign_size,offset + 8)

        # uint32_t
        num_classes = self._read_uint32(offset + 12)
        print('num_classes', num_classes,offset + 12)

        # Offset to the class index structure. The offset must point to a structure in ClassIndex format.
        # uint32_t
        class_idx_off = self._read_uint32(offset + 16)
        print('class_idx_off', class_idx_off,offset + 16)

        # Number of line number programs in the file.
        # Also this is the number of elements in the LineNumberProgramIndex structure.
        # uint32_t
        num_lnps = self._read_uint32(offset + 20)
        print('num_lnps', num_lnps, offset,offset + 20)

        # Offset to the line number program index structure.
        # The offset must point to a structure in LineNumberProgramIndex format.
        # lnp_idx_off
        lnp_idx_off = self._read_uint32(offset + 24)
        print('lnp_idx_off', lnp_idx_off, offset,offset + 24)

        # 	Number of literalArrays defined in the file.
        # 	Also this is the number of elements in the LiteralArrayIndex structure.
        # uint32_t
        num_literal_arrays = self._read_uint32(offset + 28)
        print('num_literal_arrays', num_literal_arrays, offset + 28)

        # Offset to the literalarray index structure.
        # The offset must point to a structure in LiteralArrayIndex format.
        # uint32_t
        literal_array_idx_off = self._read_uint32(offset + 32)
        print('literal_array_idx_off', literal_array_idx_off, offset + 32)

        # Number of the index regions in the file.
        # Also this is the number of elements in the RegionIndex structure.
        # uint32_t
        num_index_regions = self._read_uint32(offset + 36)
        print('num_index_regions', num_index_regions, offset + 36)

        # Offset to the index section.
        # The offset must point to a structure in RegionIndex format.
        # uint32_t
        index_section_off = self._read_uint32(offset + 40)
        print('index_section_off', index_section_off, offset + 40)

        self.pos = offset+44
        print("Final offset is : ",self.pos)
        return [magic, check_sum, version, file_size, foreign_off, foreign_size, num_classes, class_idx_off,
                num_lnps, lnp_idx_off, num_literal_arrays, literal_array_idx_off, num_index_regions,
                index_section_off]

    def _read_string(self, offset, length):
        data = self.buf[offset:offset + length]
        return data.decode('utf-8').replace('\x00', '')

    def _read_uint32(self, offset):
        return struct.unpack('I', self.buf[offset:offset + 4])[0]

    def is_valid(self):
        return self.magic == 'PANDA'


with open('modules.abc','rb') as f:
    buf = f.read()
    header = AbcHeader(buf)
    f.close()


import copy
import json
import os
import zipfile
from io import BytesIO
from typing import Any, Dict, List

from ohre.core import oh_common, oh_hap
from ohre.misc import Log


class oh_app(oh_common.oh_package):
    def __init__(self, value):
        Log.debug(f"oh_app init {type(value)}")
        if (isinstance(value, str)):
            if (not value.endswith(".app")):
                raise oh_common.ParaNotValid("Not a valid app type, must .app")
        super().__init__(value)
        self.haps = dict()
        for fname in self.package.namelist():
            if (len(fname) > 4 and fname.endswith(".hap")):
                zfiledata = BytesIO(self.package.read(fname))
                hap = oh_hap.oh_hap(zfiledata)
                self.haps[fname] = hap

        self.haps_files_path = dict()
        for hap_name, hap in self.get_haps_dict().items():
            self.haps_files_path[hap_name] = list()
            for fpath in hap.get_files():
                self.haps_files_path[hap_name].append(fpath)

    def get_files_in_app(self) -> List[str]:
        # return files's name only in app
        return self.files

    def get_files_in_haps(self) -> Dict[str, List]:
        # return files's name in all haps
        return self.haps_files_path

    def get_files(self) -> List[str]:
        # return files's name in app and all haps
        ret = copy.deepcopy(self.files)
        for hap_name, f_list in self.haps_files_path.items():
            ret.extend([os.path.join(hap_name, f_path) for f_path in f_list])
        return ret

    def get_file(self, filename: str) -> bytes:
        dir_name = os.path.dirname(filename)
        if (not dir_name):
            return super().get_file(filename)
        while (os.path.dirname(dir_name)):
            dir_name = os.path.dirname(dir_name)
        f_path = filename[len(dir_name)+1:]
        Log.debug(f"app get_file in hap {dir_name} {f_path}")
        if (dir_name in self.get_haps_name()):
            return self.haps[dir_name].get_file(f_path)

    def extract_all_to(self, unzip_folder: str, unzip_sub_hap=True) -> bool:
        try:
            self.package.extractall(unzip_folder)
            if (unzip_sub_hap):
                for root, dirs, files in os.walk(unzip_folder):
                    for fname in files:
                        if (len(fname) > 4 and fname.endswith(".hap")):
                            Log.info(f"{self.sha1} extract hap {fname} to {unzip_folder}")
                            oh_common.extract_local_zip_to(
                                os.path.join(unzip_folder, fname),
                                os.path.join(unzip_folder, f"{oh_common.HAP_EXTRACT_PREFIX}{fname[: -4]}"))
            return True
        except zipfile.BadZipFile:
            Log.warn(f"{self.sha1} Bad ZIP file, {self.file_path}")
            return False

    def get_haps_name(self) -> List[str]:
        return list(self.haps.keys())

    def get_haps_dict(self) -> Dict[str, oh_hap.oh_hap]:
        return self.haps

    def get_haps(self) -> List[oh_hap.oh_hap]:
        return list(self.haps.values())

    def get_hap(self, name: str) -> oh_hap.oh_hap:
        if (name in self.haps.keys()):
            return self.haps[name]
        else:
            return None

    def is_certificated(self) -> bool:
        pass

    def filters_filename_white_all_haps(self, rules: Dict) -> Dict:
        not_white_files_dict = dict()
        Log.info(f"{self.sha1} all haps filter white: {rules}")
        for fname, hap in self.haps.items():
            not_white_files_dict[fname] = hap.filters_filename_white(rules)
        return not_white_files_dict

    def filters_filename_black_all_haps(self, rules: Dict) -> Dict:
        not_black_files_dict = dict()
        Log.info(f"{self.sha1} all haps filter black: {rules}")
        for fname, hap in self.haps.items():
            not_black_files_dict[fname] = hap.filters_filename_black(rules)
        return not_black_files_dict

    def filters_filename_white_app_level(self, rules: Dict) -> List:
        # not filter files in haps, only scan app
        not_white_files = list()
        Log.info(f"{self.sha1} app level filter white: {rules}")
        if ("." in rules.keys()):
            pattern_list = rules["."]
            for fpath in self.files:
                if (os.sep not in fpath):  # only level-1 files
                    Log.debug(f"app level filter white: analyzing {fpath}")
                    if (not oh_common.fname_in_pattern_list(fpath, pattern_list)):
                        not_white_files.append(fpath)
        return sorted(list(set(not_white_files)))

    def filters_filename_black_app_level(self, rules: Dict) -> List:
        # not filter files in haps, only scan app
        pass



import logging
import os
from logging.handlers import RotatingFileHandler
import platform
import datetime

g_log = None
DEBUG_LOCAL = True
DEBUG_LEN = 500


def debug_print(logstr: str, level: str = "debug"):
    if (DEBUG_LOCAL and len(logstr)):
        if (len(logstr) >= DEBUG_LEN):
            print("[LOG]", level, datetime.datetime.now().strftime(
                "%Y-%m-%d %H:%M:%S"), logstr[:DEBUG_LEN], " ... truncated")
        else:
            print("[LOG]", level, datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), logstr)


def init_log(log_name: str, log_dir=""):
    global g_log
    if (platform.system() == "Windows"):
        LOG_DIR = os.path.join("D:\\ohre", "log")
    elif (platform.system() == "Linux"):
        LOG_DIR = os.path.join("/data", "ohre", "log")
    elif (platform.system() == "Darwin"):
        LOG_DIR = os.path.join("/", "Users", "Shared", "ohre", "log")
    else:
        print("Log init: NOT SUPPORTED OS")

    if (len(log_dir)):
        LOG_DIR = log_dir

    if not os.path.exists(LOG_DIR):
        os.makedirs(LOG_DIR)
    g_log = logging.getLogger(log_name)
    log_file = os.path.join(LOG_DIR, log_name + ".log")
    handle = RotatingFileHandler(log_file, mode="a", maxBytes=10 * 1024 * 1024,
                                 backupCount=10, encoding="utf-8", delay=0)
    g_log.setLevel(logging.INFO)
    formatter = logging.Formatter("%(asctime)s %(levelname)s %(message)s")
    handle.setFormatter(formatter)
    g_log.addHandler(handle)
    return g_log


def get_logger():
    global g_log
    if g_log is None:
        return init_log("default_log_name")
    return g_log


def set_debug_print_flag(print_flag: bool):
    global DEBUG_LOCAL
    DEBUG_LOCAL = print_flag


def debug(logstr, print_flag=True):
    if (print_flag and get_logger().getEffectiveLevel() <= logging.DEBUG):
        debug_print(logstr, "debug")
    g_log.debug(logstr)


def info(logstr, print_flag=True):
    if (print_flag and get_logger().getEffectiveLevel() <= logging.INFO):
        debug_print(logstr, "info")
    g_log.info(logstr)


def warn(logstr, print_flag=True):
    if (print_flag and get_logger().getEffectiveLevel() <= logging.WARNING):
        debug_print(logstr, "warn")
    g_log.warning(logstr)


def error(logstr, print_flag=True):
    if (print_flag and get_logger().getEffectiveLevel() <= logging.ERROR):
        debug_print(logstr, "error")
    g_log.error(logstr)


def critical(logstr, print_flag=True):
    if (print_flag and get_logger().getEffectiveLevel() <= logging.CRITICAL):
        debug_print(logstr, "criti")
    g_log.critical(logstr)


if __name__ == "__main__":
    init_log("Log_TEST_started_from_main")



import fnmatch
import hashlib
import json
import os
import zipfile
from typing import Any, Dict, List

import yara

from ohre.core import oh_common
from ohre.misc import Log
from ohre.res_analyzer.oh_resbuf import ResIndexBuf


class oh_hap(oh_common.oh_package):
    def __init__(self, value):
        Log.debug(f"oh_hap init {type(value)}")
        if (isinstance(value, str)):
            if (not value.endswith(".hap")):
                raise oh_common.ParaNotValid("Not a valid hap type, must be .hap")
        super().__init__(value)
        self.res_index = None

    def filter_filename_white(self, path: str, pattern_list: List) -> List:
        # path: a path prefix to specify the dir to be scanned
        # "*": all files include sub path and root path of this hap
        # pattern_list: file name pattern that are allowed in the corressponding path
        not_white_files = []
        for fpath in self.files:
            if (fpath.startswith(path) or path == "*"):
                Log.debug(f"{self.sha1} filter white in {path}: fpath patt {fpath} {pattern_list}")
                if (not oh_common.fname_in_pattern_list(os.path.basename(fpath), pattern_list)):
                    not_white_files.append(fpath)
        return sorted(not_white_files)

    def filters_filename_white(self, rules: Dict) -> List:
        # ".": level 1 files(NOT in a sub folder)
        # here, a filter means a k,v in rules dict. k: path, v: white filename pattern list
        # NOTE: As long as 1 whitelist rule hit, the file will be considered as non-whitelisted.
        # e.g. rules is {"*": ["*.png"], ".": ["pack.json"]}. then pack.json is a non-whitelisted file here
        not_white_files = []
        for path, pattern_list in rules.items():
            if (path == "."):
                continue
            l = self.filter_filename_white(path, pattern_list)
            not_white_files.extend(l)
        if ("." in rules.keys()):
            pattern_list = rules["."]
            for fpath in self.files:
                if (os.sep not in fpath):
                    Log.debug(f"{self.sha1} filter white in . : fpath patt {fpath} {pattern_list}")
                    if (not oh_common.fname_in_pattern_list(fpath, pattern_list)):
                        not_white_files.append(fpath)
        return sorted(list(set(not_white_files)))

    def filter_filename_black(self, path: str, pattern_list: List) -> List:
        black_files = []
        for fpath in self.files:
            if (fpath.startswith(path) or path == "*"):
                for patt in pattern_list:
                    Log.debug(f"{self.sha1} filter black in {path}: fpath patt {fpath} {patt}")
                    if (fnmatch.fnmatch(os.path.basename(fpath), patt)):
                        black_files.append(fpath)
        return sorted(black_files)

    def filters_filename_black(self, rules: Dict) -> List:
        black_files = []
        for path, pattern_list in rules.items():
            if (path == "."):
                continue
            l = self.filter_filename_black(path, pattern_list)
            black_files.extend(l)
        if ("." in rules.keys()):
            pattern_list = rules["."]
            for fpath in self.files:
                if (os.sep not in fpath):
                    for patt in pattern_list:
                        Log.debug(f"{self.sha1} filter black in . : fpath patt {fpath} {patt}")
                        if (fnmatch.fnmatch(fpath, patt)):
                            black_files.append(fpath)
        return sorted(list(set(black_files)))

    def get_resources_index_raw(self) -> bytes:
        if ("resources.index" not in self.files):
            Log.error(f"{self.sha1} resources.index NOT found in this hap")
            return bytes()
        else:
            return super().get_file("resources.index")

    def get_resources_index(self) -> Dict:
        if (self.res_index is None):
            buf = self.get_resources_index_raw()
            res_content = ResIndexBuf(buf)
            self.res_index = res_content.resources_content
        return self.res_index

    def compare_files_with_resources_index(self) -> Dict:
        if (self.res_index is None):
            self.get_resources_index()

        self.get_module_json()  # TODO: This might not exist if not called

        # TODO: Verify if the prefix is correct
        files_prefix = "resources"
        resources_prefix = os.path.join(self.get_module_package_name(), "resources")

        # TODO: We need a better way to pre-filter, e.g., using "file_type"
        res_index_list = [res["file_value"] for res in self.res_index["resource_items"].values()]

        def filter_path(prefix, path_list):
            import pathlib
            return [
                path.relative_to(prefix).as_posix()
                for p in path_list if (path := pathlib.Path(p)).is_relative_to(prefix)
            ]

        res_list = filter_path(resources_prefix, res_index_list)
        files_list = filter_path(files_prefix, self.files)

        return {"files_only": list(set(files_list) - set(res_list)),
                "resource_index_only": list(set(res_list) - set(files_list)),
                "intersection ": list(set(res_list) & set(files_list))}

    # === module.json analysis START ===

    def get_module_json_raw(self) -> bytes:
        if ("module.json" not in self.files):
            Log.error(f"{self.sha1} module.json NOT found in this hap")
            return bytes()
        else:
            return super().get_file("module.json")

    def get_module_json(self) -> Dict:
        if ("module.json" not in self.files):
            Log.error(f"{self.sha1} module.json NOT found in this hap")
            return dict()
        json_string = self.get_file("module.json").decode("utf-8", errors="ignore")
        self.module_json_dict = json.loads(json_string)
        return self.module_json_dict

    def get_module_name(self) -> str:
        if ("module" in self.module_json_dict and "name" in self.module_json_dict["module"]):
            return self.module_json_dict["module"]["name"]
        return ""

    def get_module_package_name(self) -> str:
        if ("module" in self.module_json_dict and "packageName" in self.module_json_dict["module"]):
            return self.module_json_dict["module"]["packageName"]
        return ""

    def get_module_device_types(self) -> List:
        if ("module" in self.module_json_dict and "deviceTypes" in self.module_json_dict["module"]):
            return self.module_json_dict["module"]["deviceTypes"]
        return ""
    # === module.json analysis END ===
